#! /bin/env python

import numpy as np
import matplotlib.pylab as plt
import pacs
import pacsplot as pp

def distancermslist(col,array,antposfile,project,cFreq,sFreq):
    """
        The purpose of this function is to take a column input for the carmaphaseDiffs.txt
    and szaphaseDiffs.txt files and output 2 lists.
    Input:
    col: The column index of the phaseDiff file that the user wants to access. This col index corresponds
    to a baseline pair of the format num1-num2. For example: "1-2"

    Outputs:
    diffStdlist: A list of differences of standard deviations for the baseline that corresponds to the inputted column.
    The difference is: Std(CARMA) - Std(CARMA-SZA*3). So for each of these differences, if the difference is positive,
    there was improvement after correction, and if the difference is negative, then there was not improvement.
    cblLen: The baseline length of the carma baseline that corresponds to the inputted column
    """
    scalar = float(cFreq)/float(sFreq)

    import carmatable as ct
    tableInc = ct.CarmaTable()
    tableInc.open("carmaphaseDiffs.txt")
    tableIns = ct.CarmaTable()
    tableIns.open("szaphaseDiffs.txt")
    times = tableInc.getCol(0)
    cColdata = tableInc.getCol(col)
    sColdata = tableIns.getCol(col)
    cHeaderwords = tableInc.getColNames()
    sHeaderwords = tableIns.getColNames()

    sColdataScaled = []
    for element in sColdata:
        sColdataScaled.append(int(element) * scalar)
    
    rmslistCarma = [] #list of lists
    rmslistCorrected = [] #list of lists
    index = []
    count = 1

    ################ START TIMES AND END TIMES #########
    threshold = 15 #number of seconds that will cause program to think that time interval is a new one
    starttimes = []
    endtimes = []

    oldTime = "00:00:00.0"
    for time in times:
        if oldTime == "00:00:00.0":
            starttimes.append(oldTime)
        elif (pp.timeToSec(time) - pp.timeToSec(oldTime)) > threshold:
            endtimes.append(oldTime)
            starttimes.append(time)
        oldTime = time
    endtimes.append(time)
    
    for time in starttimes:
        rmslistCarma.append([])
        rmslistCorrected.append([])
        index.append(count)
        count = count + 1

    for (time,c,s) in zip(times,cColdata,sColdataScaled):
        count = 0
        for (sTime,eTime) in zip(starttimes,endtimes):
            if (sTime < time) and (eTime > time):
                rmslistCarma[count].append(int(c))
                rmslistCorrected[count].append(int(c) - int(s))
            count = count + 1
    
    stdCarmalist = []
    stdCorrectedlist = []
    for (list1,list2) in zip(rmslistCarma,rmslistCorrected):
        stdCarmalist.append(pp.stdev(list1))
        stdCorrectedlist.append(pp.stdev(list2))
    #Calculation of Std's

    diffStdlist = []
    for (stdC,stdCo) in zip(stdCarmalist,stdCorrectedlist):
        diffStdlist.append(stdC - stdCo)
    #calculation of the difference in Std's

    ###BASELINE LENGTH CALC
    #print "Calculating baseline length for: " + cHeaderwords[col]
    
    #cblLen = pp.baselineLength(cHeaderwords[col], "carma")
    cblLen = pp.baselineLength(cHeaderwords[col], "carma",antposfile)
    #######################

    return (diffStdlist,cblLen)

def plot(array,antposfile,project,cFreq,sFreq):
    scalar = float(cFreq)/float(sFreq)
    
    plt.figure(num=1, figsize=(18.5,11), dpi=90)
    count = 1

    distancelist = [] #list that matches outer list of improvement list
    improvementlist = [] #list of lists
    while (count <= 28):
        (curDifflist, curDistance) = distancermslist(count,array,antposfile,project,cFreq,sFreq)
        improvementlist.append(curDifflist)
        distancelist.append(curDistance)
        count = count + 1

    title = "Distance vs RMS Plot for %s"%(project)
    for (values,distance) in zip(improvementlist,distancelist):
        distanceplotlist = []
        for value in values:
            distanceplotlist.append(distance)
        plt.scatter(distanceplotlist, values, s=100, marker='^', c='g')
        #for distance in distanceplotlist:   #vertical lines
        #    plt.plot([distance,distance], [plt.axis()[2],plt.axis()[3]], 'k:')
        plt.plot([plt.axis()[0],plt.axis()[1]], [0,0], 'k-') #horizontal y=0 line
        plt.title(title)
        plt.xlabel('Baseline Distance (meters)')
        plt.ylabel('Difference in Std: (Std(CARMA) - Std(CARMA-SZA*%.3f))'%(scalar))
        
    plt.savefig("distancerms001.pdf", orientation='landscape')
    plt.close(1)
    #plt.show()



