#! /bin/env python

import numpy as np
import matplotlib.pylab as plt
import pacs
import pacsplot as pp
import carmatable as ct
import baselineplot as blp

#provide:  rmsplot,plots, rmsplotOld

def rmsplot(col,array,antposfile,project,cFreq,sFreq,file):
    scalar = float(cFreq)/float(sFreq)
    tableInc = ct.CarmaTable()
    tableInc.open("carmaphaseDiffs.txt")
    tableIns = ct.CarmaTable()
    tableIns.open("szaphaseDiffsInterpolated.txt")
    times = tableInc.getCol(0)
    sTimes=tableIns.getCol(0)
    cColdata = tableInc.getCol(col)
    sColdata = tableIns.getCol(col)
    cHeaderwords = tableInc.getColNames()
    sHeaderwords = tableIns.getColNames()
    for i in range(len(times)-1,-1,-1):
        if times[i] not in sTimes:
            del times[i]
            del cColdata[i]
    times=[pp.timeToSec(time) for time in times]
    sColdataScaled = []
    for element in sColdata:
        sColdataScaled.append(float(element) * scalar)
    
    rmslistCarma = [] #list of lists
    rmslistCorrected = [] #list of lists
    index = []
    count = 1

    ################ START TIMES AND END TIMES #########
    threshold = 15 #number of seconds that will cause program to think that time interval is a new one
    intervals=[]
    for this,prev in zip(times[1:],times[:-1]):
        if this-prev>threshold:
            intervals.append((prev,this))
    starttimes = [times[0]]
    endtimes = []
    for start,end in intervals:
        starttimes.append(end)
        endtimes.append(start)
    endtimes.append(times[-1])

    ###Start times are when the ranges start, end is same
    ###Use secs past midnight insted of runtime
    for time in starttimes:
        rmslistCarma.append([])
        rmslistCorrected.append([])
        index.append(count)
        count = count + 1

    for (time,c,s) in zip(times,cColdata,sColdataScaled):
        count = 0
        for (sTime,eTime) in zip(starttimes,endtimes):
            if (sTime < time) and (eTime > time):
                rmslistCarma[count].append(int(c))
                rmslistCorrected[count].append(int(c) - int(s))
            count = count +  1
    
    #
    stdCarmalist = []
    stdCorrectedlist = []
    for (list1,list2) in zip(rmslistCarma,rmslistCorrected):
        stdCarmalist.append(pp.stdev(list1))
        stdCorrectedlist.append(pp.stdev(list2))
    #######

    ###BASELINE LENGTH CALC

    #cblLen = pp.baselineLength(cHeaderwords[col], "carma")
    sblLen = pp.baselineLength(sHeaderwords[col], "sza")
    cblLen = pp.baselineLength(cHeaderwords[col], "carma",antposfile)
    #sblLen = pp.baselineLength(sHeaderwords[col], "sza",antposfile)
    #######

    title = "CARMA " + cHeaderwords[col] + "(" + str(int(cblLen)) + " m) and SZA " + sHeaderwords[col] + "(" + str(int(sblLen)) + " m) RMS"

    #title = "CARMA " + cHeaderwords[col] + " and SZA " + sHeaderwords[col] + " RMS Plot"

    #plt.plot(index, stdCarmalist, 'g^')
    #plt.plot(index, stdCorrectedlist, 'bv')

    plt.scatter(index, stdCarmalist, s=150, marker='^', c='g')
    plt.scatter(index, stdCorrectedlist, s=50, marker='v', c='b')
    plt.axis([plt.axis()[0],plt.axis()[1],plt.axis()[2],plt.axis()[3]+20])
    plt.legend(('Std(CARMA)','Std(CARMA-SZA*%.3f)'%(scalar)), 'best')
    plt.plot([index,index], [plt.axis()[2],plt.axis()[3]], 'k:') #makes a line as tall as current axis
    plt.title(title, fontsize=12)
    plt.xlabel('Time Interval')
    plt.ylabel('Stdev of phase (degrees)')

    ###
    names = ["Index","CARMA RMS","Corrected RMS"]
    types = ["i", "d", "d"]
    units = ["","",""]
    tsources = ct.CarmaTable(names,types,units)

    dict = {"cbl":cHeaderwords[col],"cblLen":str(int(cblLen)),"sbl":sHeaderwords[col],"sblLen":str(int(sblLen))}
    tsources.setDict(dict)
    
    tsources.editCol(index,0)
    tsources.editCol(stdCarmalist,1)
    tsources.editCol(stdCorrectedlist,2)
    tsources.write(file+"%03i.dat"%(col))
    ###Editing sources and times file to add RMS

def plots(array,project,cFreq,sFreq,file="finalTimes/finalCorrelation"):

    plt.figure(num=1)
    plt.close(1)
    
    count = 1
    plt.figure(num=1, figsize=(18.5,11), dpi=90)
    while (count <= 6):
        plt.subplot(2,3,count)
        rmsplot(count, array, project, cFreq, sFreq, file)
        count = count + 1
    #plots first 6 plots
    plt.savefig("rmstime001.pdf", orientation='landscape')

    plt.figure(num=2, figsize=(18.5,11), dpi=90)
    while (count <= 12):
        plt.subplot(2,3,count-6)
        rmsplot(count, array, project, cFreq, sFreq, file)
        count = count + 1
    #plots 7-12
    plt.savefig("rmstime002.pdf", orientation='landscape')

    plt.figure(num=3, figsize=(18.5,11), dpi=90)
    while (count <= 18):
        plt.subplot(2,3,count-12)
        rmsplot(count, array, project, cFreq, sFreq, file)
        count = count + 1
    #plots 13-18
    plt.savefig("rmstime003.pdf", orientation='landscape')

    plt.figure(num=4, figsize=(18.5,11), dpi=90)
    while (count <= 24):
        plt.subplot(2,3,count-18)
        rmsplot(count, array, project, cFreq, sFreq, file)
        count = count + 1
    #plots 19-24
    plt.savefig("rmstime004.pdf", orientation='landscape')

    plt.figure(num=5, figsize=(18.5,11), dpi=90)
    while (count <= 28):
        plt.subplot(2,3,count-24)
        rmsplot(count, array, project, cFreq, sFreq, file)
        count = count + 1
    #plots 25-28
    plt.savefig("rmstime005.pdf", orientation='landscape')

    plt.close(1)
    plt.close(2)
    plt.close(3)
    plt.close(4)
    plt.close(5)

def rmsplotOld(col,array,antposfile,project,cFreq,sFreq,file):
    scalar = float(cFreq)/float(sFreq)
    
    tableInc = ct.CarmaTable()
    tableInc.open("carmaphaseDiffs.txt")
    tableIns = ct.CarmaTable()
    tableIns.open("szaphaseDiffs.txt")
    times = tableInc.getCol(0)
    cColdata = tableInc.getCol(col)
    sColdata = tableIns.getCol(col)
    cHeaderwords = tableInc.getColNames()
    sHeaderwords = tableIns.getColNames()

    sColdataScaled = []
    for element in sColdata:
        sColdataScaled.append(int(element) * scalar)
    
    rmslistCarma = [] #list of lists
    rmslistCorrected = [] #list of lists
    index = []
    count = 1

    ################ START TIMES AND END TIMES #########
    threshold = 15 #number of seconds that will cause program to think that time interval is a new one
    starttimes = []
    endtimes = []
    ################CONVERT FROM RUNTIME
    ###Start times are when the ranges start, end is same
    ###Use secs past midnight insted of runtime
    oldTime = "00:00:00.0"
    for time in times:
        if oldTime == "00:00:00.0":
            starttimes.append(oldTime)
        elif (pp.timeToSec(time) - pp.timeToSec(oldTime)) > threshold:
            endtimes.append(oldTime)
            starttimes.append(time)
        oldTime = time
    endtimes.append(time)
    ###################
    for time in starttimes:
        rmslistCarma.append([])
        rmslistCorrected.append([])
        index.append(count)
        count = count + 1

    for (time,c,s) in zip(times,cColdata,sColdataScaled):
        count = 0
        for (sTime,eTime) in zip(starttimes,endtimes):
            if (sTime < time) and (eTime > time):
                rmslistCarma[count].append(int(c))
                rmslistCorrected[count].append(int(c) - int(s))
            count = count +  1
    
    #
    stdCarmalist = []
    stdCorrectedlist = []
    for (list1,list2) in zip(rmslistCarma,rmslistCorrected):
        stdCarmalist.append(pp.stdev(list1))
        stdCorrectedlist.append(pp.stdev(list2))
    #######

    ###BASELINE LENGTH CALC

    cblLen = pp.baselineLength(cHeaderwords[col], "carma")
    sblLen = pp.baselineLength(sHeaderwords[col], "sza")
    #cblLen = pp.baselineLength(cHeaderwords[col], "carma",antposfile)
    #sblLen = pp.baselineLength(sHeaderwords[col], "sza",antposfile)
    #######

    title = "CARMA " + cHeaderwords[col] + "(" + str(int(cblLen)) + " m) and SZA " + sHeaderwords[col] + "(" + str(int(sblLen)) + " m) RMS"

    #title = "CARMA " + cHeaderwords[col] + " and SZA " + sHeaderwords[col] + " RMS Plot"

    #plt.plot(index, stdCarmalist, 'g^')
    #plt.plot(index, stdCorrectedlist, 'bv')

    plt.scatter(index, stdCarmalist, s=150, marker='^', c='g')
    plt.scatter(index, stdCorrectedlist, s=50, marker='v', c='b')
    plt.axis([plt.axis()[0],plt.axis()[1],plt.axis()[2],plt.axis()[3]+20])
    plt.legend(('Std(CARMA)','Std(CARMA-SZA*%.3f)'%(scalar)), 'best')
    plt.plot([index,index], [plt.axis()[2],plt.axis()[3]], 'k:') #makes a line as tall as current axis
    plt.title(title, fontsize=12)
    plt.xlabel('Time Interval')
    plt.ylabel('Stdev of phase (degrees)')

    ###
    names = ["Index","CARMA RMS","Corrected RMS"]
    types = ["i", "d", "d"]
    units = ["","",""]
    tsources = ct.CarmaTable(names,types,units)

    dict = {"cbl":cHeaderwords[col],"cblLen":str(int(cblLen)),"sbl":sHeaderwords[col],"sblLen":str(int(sblLen))}
    tsources.setDict(dict)
    
    tsources.editCol(index,0)
    tsources.editCol(stdCarmalist,1)
    tsources.editCol(stdCorrectedlist,2)
    tsources.write(file+"%03i.dat"%(col))
    ###Editing sources and times file to add RMS

def plots(array,antpos,project,cFreq,sFreq,file="finalTimes/finalCorrelation"):

    plt.figure(num=1)
    plt.close(1)
    
    count = 1
    plt.figure(num=1, figsize=(18.5,11), dpi=90)
    while (count <= 6):
        plt.subplot(2,3,count)
        rmsplot(count, array, antpos, project, cFreq, sFreq, file)
        count = count + 1
    #plots first 6 plots
    plt.savefig("rmstime001.pdf", orientation='landscape')

    plt.figure(num=2, figsize=(18.5,11), dpi=90)
    while (count <= 12):
        plt.subplot(2,3,count-6)
        rmsplot(count, array, antpos, project, cFreq, sFreq, file)
        count = count + 1
    #plots 7-12
    plt.savefig("rmstime002.pdf", orientation='landscape')

    plt.figure(num=3, figsize=(18.5,11), dpi=90)
    while (count <= 18):
        plt.subplot(2,3,count-12)
        rmsplot(count, array, antpos, project, cFreq, sFreq, file)
        count = count + 1
    #plots 13-18
    plt.savefig("rmstime003.pdf", orientation='landscape')

    plt.figure(num=4, figsize=(18.5,11), dpi=90)
    while (count <= 24):
        plt.subplot(2,3,count-18)
        rmsplot(count, array, antpos, project, cFreq, sFreq, file)
        count = count + 1
    #plots 19-24
    plt.savefig("rmstime004.pdf", orientation='landscape')

    plt.figure(num=5, figsize=(18.5,11), dpi=90)
    while (count <= 28):
        plt.subplot(2,3,count-24)
        rmsplot(count, array, antpos, project, cFreq, sFreq, file)
        count = count + 1
    #plots 25-28
    plt.savefig("rmstime005.pdf", orientation='landscape')

    plt.close(1)
    plt.close(2)
    plt.close(3)
    plt.close(4)
    plt.close(5)

