"""By Roger Curley
Last updated 8/24/09"""
import sys
import os
import re
import carmatable as ct
import math
import parfile
def angular(ra1,dec1,ra2,dec2):
   """   computes the angular distance between two objects, in degrees
   input ra,dec in decimal Hours and Degrees,i.e.   ra is 0..24
   and dec is -90..90
   Wikipedia list 3 different formulae, a generic one, one better for small
   angles, one better for dealing with close to pole object
   See also:    http://en.wikipedia.org/wiki/Great-circle_distance
   """
   #print "ra1:%f dec1:%f ra2:%f dec2:%f"%(ra1,dec1,ra2,dec2)
   d2r = math.pi/180.0
   r1 = ra1*15*d2r#lamda
   r2 = ra2*15*d2r
   d1 = dec1*d2r#rotated null
   d2 = dec2*d2r
   #print "r1:%f d1:%f r2:%f d2:%f"%(r1,d1,r2,d2)
   #return math.acos(math.sin(d1)*math.sin(d2)+math.cos(d1)*math.cos(d2)*math.cos(r1-r2))/d2r
   return 2.0*math.asin(math.sqrt(pow(math.sin((d1-d2)/2.0),2.0)+math.cos(d1)*math.cos(d2)*pow(math.sin(r1-r2),2.0)))
def rd_convert(rd):
    """convert string to (r,d) pair in degrees
    """
    # rd = "12 33 44.5 -3 12 55.32"
    s = rd.split()
    ra = float(s[0]) + float(s[1])/60.0 + float(s[2])/3600.0
    if s[3][0] == '-' :
        dec = float(s[3]) - float(s[4])/60.0 - float(s[5])/3600.0
    else:
        dec = float(s[3]) + float(s[4])/60.0 + float(s[5])/3600.0
    return (ra,dec)
def compareRadec(rad1,rad2,tolerance):
    """Returns a boolean of if the two RA-DECs are within tolerance of each other."""
    return rad1==rad2 or angular(rd_convert(rad1)[0],rd_convert(rad1)[1],rd_convert(rad2)[0],rd_convert(rad2)[1])<tolerance

def sillyTimes(vis=None):
   """this function doesn't do anything yet, but is meant to return the source names
   and time ranges that findCommonTimes() needs?
   We use listobs, but listobs does not report the final time.
   uvindex does appear to do this, but its times do not match those of listobs.
   aipy seems ok as well.
   Also note that currently listobs on CARMA data differ from uvlist times:
   listobs uses the 'ut', which is inttime/2 larger than time.
   This is NOT true for SZA data,  where they are the same. TBD if they
   are at the start of mid-interval
   To go back to raw times... this one also seems to work:

   uvio <FILE> | grep DATA | egrep -w '(source|time)'
   ...
   0x00000648 DATA: source    URANUS
   0x000014f0 DATA: time               2454864.495   09FEB01:23:52:34.0
   ...
   0x00c2ef68 DATA: source    3C111   
   0x00c2f918 DATA: time               2454864.501   09FEB02:00:00:44.0
   ...
   0x0606e520 DATA: time               2454864.567   09FEB02:01:36:39.5
   """
   cmd = "uvio %s | grep DATA | egrep -w '(source|inttime|time)'" % vis
   return None
def readListobs(isCarma,file,pipeDef=None):
    """
    This function reads in data from listobs and converts it to a format usable by findCommonTimes
    Inputs:
        isCarma: Boolean for CARMA/SZA
        file: The location of the miriad file
        pipeDef: A file containing variables to be used later.
            This will add the frequency of SZA and CARMA's observations. (cFreq,sFreq)
    """
    names=[]
    os.system("rm link>& /dev/null")
    os.system("ln -s "+file+" link")
    if isCarma:
        os.system("listobs vis=link log=listobs.car")
        fin=open("listobs.car")
    else:
        os.system("listobs vis=link log=listobs.sza")
        fin=open("listobs.sza")
    text=fin.read()
    if fin.close():#In case of listobs crashing...
        os.system("rm link>& /dev/null")
        raise listobsError()
    if pipeDef:
        if isCarma:
            print "Setting cFreq to %.1f"%float(re.search(r"(?<=First LO:) *\d*\.\d*",text).group(0))
            pipeDef.set("cFreq",float(re.search(r"(?<=First LO:) *\d*\.\d*",text).group(0)))
            pipeDef.set("date",re.search(r"(?<=Chronology of Observations on) *\d\d\w\w\w\d\d",text).group(0).strip())
        else:
            pipeDef.set("sFreq",30.0)#SZA frequencies are not logged at this time.
    rad=re.findall(r"-?\d?\d \d\d \d\d\.\d\d +-?\d?\d \d\d \d\d\.\d\d",text)#Generates a list of RA-DECS
    #Generates a list of observation targets
    namekey=re.findall(r"(?<=\n)[^ ]+(?= +\w+ +-?\d?\d \d\d \d\d\.\d\d +-?\d?\d \d\d \d\d\.\d\d)",text)
    namerad=zip(rad,namekey)
    namerad.sort()
    nameradb=[]
    #nameradb will have consolidated duplicate RA-DECs and removed noise
    if namerad[-1][1]!="NOISE":
        nameradb.append(namerad[-1])
    for this,prev in map(None,namerad[1:],namerad[:-1]):
        if this[0]==prev[0] and this[1]!="NOISE":
            text=text.replace(prev[0][1],this[0][1])
        else:
            if prev[1]!="NOISE":
                nameradb.append(prev)
    text=text.split("Source              UT      LST     Dur Elev       Sys Temps (K)")[1]
    times=re.findall(r"\d\d\d\d\d\d\.\d",text)[::2]#Returns a list of observation times.
    endDuration=float(re.findall(r"(?<=\d{6}\.\d) *\d+.\d",text)[-1])
    #The final interval requires special calculation, based on observation duration, to
    #   find the end time, as there is no next observation to use for the end time.
    endTime=[float(times[-1][0:2]),float(times[-1][2:4]),float(times[-1][4:])]
    endTime[2]+=(endDuration%1)*6
    endTime[1]+=int(endDuration)
    if endTime[2]>=60.:
        endTime=[endTime[0],(endTime[1]+1.),(endTime[2]-60.)]
    if endTime[1]>=60.:
        endTime=[(endTime[0]+1.),(endTime[1]-60.),endTime[2]]
    times.append("%02.0f%02.0f%04.1f"%tuple(endTime))
    for line in text.split("\n"):
        names.append(line.split(" ")[0])
    names=names[2:-2]+["END"]
    if isCarma:
        timeline=map(None,times,names,[])
    else:
        timeline=map(None,times,[],names)
    #Solving the midnight problem
    timelineb=[timeline[0]]
    for (time,car,sza) in timeline[1:]:
        if time<timelineb[-1][0]:
            timelineb.append((str((int(time[0:2])+24))+time[2:],car,sza))
        else:
            timelineb.append((time,car,sza))

    os.system("rm link>& /dev/null")
    return (nameradb,timelineb)
class listobsError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)
def findCommonTimes(file1=None,file2=None,*args,**kwargs):
    """
    This function takes a CARMA and SZA directory, and returns a list of
    intervals when the two are observing the same target.
    Required arguments:
        file1: the CARMA directory
        file2: the SZA directory
    Optional arguments:
        screenPrint= If present, output will be printed
                to the screen.
        output= a filename to write the output to
    Returns nothing if no errors, otherwise returns...
        1: CARMA listobs failed
        2: SZA listobs failed
    """
    if file2==None:
        #file2 is the last mandatory argument
        #it will always be missing if there are not
        #enough arguments.
        print findCommonTimes.__doc__
        return 3
    screenPrint="screenPrint" in args
    fout=False
    sWhitelist=None
    sBlacklist=None
    if "sourceWhitelist" in kwargs.keys():
        sWhitelist=kwargs["sourceWhitelist"]
    if "sourceBlacklist" in kwargs.keys():
        sBlacklist=kwargs["sourceBlacklist"]
    if "output" in kwargs.keys():
        output=kwargs["output"]
        fout=ct.CarmaTable(["Start time","End time","TargetC","TargetS"],["hms","hms","s","s"],["","","",""])
    if screenPrint:
        print ("Comparing %s and %s"%(file1.split('/')[-1].split(".mir")[0],file2.split('/')[-1].split(".mir")[0]))
    if fout:
        fout.addComment(-1,"Comparing %s and %s"%(file1.split('/')[-1].split(".mir")[0],file2.split('/')[-1].split(".mir")[0]))
    #Namerad is a list of elements of the form:
    #('5 10 02.37  18 00 41.59', '0510+180')
    p=None
    pipeDef=None
    if 'pipeDef' in kwargs.keys():
        pipeDef=kwargs['pipeDef']
    namerad1,timeline1=readListobs(True,file1,pipeDef)    #Read in the CARMA data from listobs
    radToName1=dict(namerad1)                             #This allows conversion from RA-DEC to CARMA name.
    namerad2,timeline2=readListobs(False,file2,pipeDef)   #Read in the SZA data from listobs
    nameTorad2={}                                         #This allows conversion from SZA name to RA-DEC.
    for rad,name in namerad2:
        nameTorad2[name]=rad
    name2ToName1={}                                 #This allows conversion from SZA name to CARMA name
    for name in nameTorad2.keys():
        if name!="NOISE":
            for rad in radToName1.keys():
                if compareRadec(rad,nameTorad2[name],1.0/3600.0):
                        name2ToName1[name]=radToName1[rad]
    timeline1.extend(timeline2)
    timeline1.sort()
    timeline3=[timeline1[0]]
    for this in timeline1[1:]:
        prev=timeline3[-1]
        if this[1]==None:
            timeline3.append((this[0],prev[1],this[2]))
        else:
            timeline3.append((this[0],this[1],prev[2]))
    timeline4=[timeline3[0]]#Consolidate continuous observations
    for (this,prev) in zip(timeline3[1:],timeline3[0:-1]):
        if  not (this[1]==prev[1] and this[2]==prev[2]):
            timeline4.append(this)
    timeline5=[]
    for this,next in map(None,timeline4[:-1],timeline4[1:]):
        timeline5.append(((this[0],next[0]),this[1],this[2]))
    timeline6=[]#Filtering out nonmatching intervals
    for (stime,etime),carName,szaName in timeline5:
        if carName!="NOISE" and szaName in name2ToName1.keys() and carName==name2ToName1[szaName]:
            timeline6.append((stime,etime,carName,szaName))
    if sWhitelist:#Filtering by white list
        timeline7=[(stime,etime,carName,szaName) for (stime,etime,carName,szaName) in timeline6\
                if carName in sWhitelist or szaName in sWhitelist]
    else: timeline7=timeline6
    if sBlacklist:#Filtering by black list
        print "Checking Blacklist"
        timeline8=[(stime,etime,carName,szaName) for (stime,etime,carName,szaName) in timeline7\
                if not(carName in sBlacklist or szaName in sBlacklist)]
    else: timeline8=timeline7
    timeline9=[]#Reconverting back to valid hhmmss
    for (stime,etime,carName,szaName) in timeline8:
        if stime!='?':
            stime=str(int(stime[0:2])%24)+stime[2:]
        if etime!='?':
            etime=str(int(etime[0:2])%24)+etime[2:]
        timeline9.append((stime,etime,carName,szaName))
    timepad=lambda x:("0"+x)[-8:]
    if screenPrint:
        print "%-15s%-15s%-15s%s"%("Start time","End time","TargetC","TargetS")
        for (start,end,carName,szaName) in timeline9:
                print "%-15s%-15s%-15s%s"%(timepad(start),timepad(end),carName,szaName)
    if fout:
        for (start,end,carName,szaName) in timeline9:
                fout.addRow([timepad(start),timepad(end),carName,szaName])
        fout.write(output)
def timebatch(root=None,outroot="./timeoutput",*args,**kwargs):
    """
    Usage: timebatch(root,[outputfile],["update"|"updateshort"],["silent"],[whitelist=__],[blacklist=__])
    Purpose: Determines the times when both SZA and the rest of CARMA are
    focused on the same target at the same time for multiple observations.
    Input:
        root: The directory to search for miriad files.
        outputfile:The file to put the results in; will be timeoutput unless specified
        "update":Specifying this will only examine datasets that don't have an
            existing output file.
        "updateshort":Specifying this will only examine datasets that don't have an
            existing output file with data in it.
        "silent":This will supress messages showing which file is being examined.
        whitelist:This specifies a text file, containing entries of the same form as
            cx252.2A_230Arp193.5. All files in root will be examined unless whitelist
            exists, in which case only files in whitelist will be used.
        blacklist:This specifies a text file, in the same format as whitelist, which
            specifies files to not be examined.
        Note that a file must pass all the filters to be examined (for example, if a
            blacklist and whitelist exist, it must be both on the whitelist and off
            the blacklist.
    """
    if root==None:
            #root is mandatory
            print timebatch.__doc__
            return 3
    allfiles=[]
    from os.path import join, getsize
    useswhitelist="whitelist" in kwargs
    usesblacklist="blacklist" in kwargs
    silent="silent" in args
    update="update" in args or "updateshort" in args
    updateshort="updateshort" in args
    if useswhitelist:
        fin=open(kwargs["whitelist"])
        whitelist=fin.read()
        fin.close()
    if usesblacklist:
        fin=open(kwargs["blacklist"])
        blacklist=fin.read()
        fin.close()
    if update:
        completelist=[]
        fin=os.popen("ls -l "+outroot)
        for line in fin.readlines()[1:]:
            if not updateshort or int(line.split()[4])>108:
                completelist.append(line.split()[-1][:-4])
        fin.close()
    for root, dirs, files in os.walk(root):
        output=[]
        for file in dirs[:]:
        #whitelist takes files in form cx252.2A_230Arp193.5
            #finds SZA files and then generates their CARMA pair because if the SZA files exist the CARMA ones will too.
            if ('.gz' not in file and 'zr' in file and file[2:-4]!="" and (not useswhitelist or file[2:-4] in whitelist)):
                if not (usesblacklist and file[2:-4] in blacklist) and not (update and file[2:-4] in completelist): 
                    output.append((file[2:]+"iad",file,root))
        allfiles=allfiles+output
    os.system("mkdir %s>& /dev/null"%(outroot))
    errors=[]
    for cx,zr,root in allfiles:
        out=(outroot+"/"+cx[:-6]+"txt")
        if not silent:
            print "%s\n\t%s\n\t%s\n\t%s"%(root,cx,zr,out)
        try:
            findCommonTimes((root+"/"+cx),(root+"/"+zr),output=out)
        except KeyboardInterrupt:
            os.system("rm "+out)
        except listobsError:
            errors.append("Listobs crashed on\n"+cx)
            if not silent:
                print "Listobs failed"
    if errors:
        print "Check errors.txt"
        fout=open("errors.txt",'w')
        fout.write("After correcting the source of the errors, this file can be used as a whitelist to fix these files.\n")
        for error in errors:
            fout.write(error+'\n')
        fout.close()
